#include <config.h>

#ifdef HAVE_DXR2

#include <stdio.h>
#include <string.h>

#include "dxr2-api.h"
#include "cache.h"

static char cache_file_name [] = "/.dxr2-overlay-settings-cache" ;
static char id1_check_string [] = "MINIDXR2" ;
static char id2_check_string [] = VERSION " " __DATE__ " " __TIME__ ;

/* The cache is tied by the compile version and date to a specific
 * version of the minidxr2 binary. This is deliberate. I don't want
 * any possibility of using an old cache and getting the settings
 * completely wrong as a result. */

static FILE * open_cache_file ( char * perm )
{
	char		filename [ FNAME_LENGTH ] ;
	char		* home_dir ;

	home_dir = getenv ( "HOME" ) ;
	if ( home_dir == NULL )
	{
		home_dir = "/" ;
	}

	if ( strlen ( home_dir ) + strlen ( cache_file_name ) >= FNAME_LENGTH_1 )
	{
		return NULL ;
	}

	strcpy ( filename , home_dir ) ;
	strcat ( filename , cache_file_name ) ;

	return fopen ( filename , perm ) ;
}


int cache_available ( dxr2_vgaParams_t * vgaBuf )
{
	FILE		* fd ;
	int			ok = 0 ;
	char		id1 [ 128 ] ;
	char		id2 [ 128 ] ;
	dxr2_vgaParams_t	vgaBuf2 ;

	fd = open_cache_file ( "rb" ) ;

	if ( fd != NULL )
	{
		if (( fread ( id1 , strlen ( id1_check_string ) , 1 , fd ) == 1 )
		&& ( fread ( & vgaBuf2 , sizeof ( dxr2_vgaParams_t ) , 1 , fd ) == 1 )
		&& ( fread ( id2 , strlen ( id2_check_string ) , 1 , fd ) == 1 )
		&& ( strncmp ( id1 , id1_check_string , strlen ( id1_check_string )) == 0 )
		&& ( strncmp ( id2 , id2_check_string , strlen ( id2_check_string )) == 0 ))
		{
			memcpy ( vgaBuf , & vgaBuf2 , sizeof ( dxr2_vgaParams_t )) ;
			ok = 1 ;
		}
		fclose ( fd ) ;
	}
	return ok ;
}


int cache_write ( dxr2_vgaParams_t * vgaBuf )
{
	FILE		* fd ;

	fd = open_cache_file ( "wb" ) ;

	if ( fd != NULL )
	{
		if (( fwrite ( id1_check_string , strlen ( id1_check_string ) , 1 , fd ) == 1 )
		&& ( fwrite ( vgaBuf , sizeof ( dxr2_vgaParams_t ) , 1 , fd ) == 1 )
		&& ( fwrite ( id2_check_string , strlen ( id2_check_string ) , 1 , fd ) == 1 ))
		{
			fclose ( fd ) ;
			return 1 ;
		}
		fclose ( fd ) ;
	}
	return 0 ;
}



#endif

