/*
 * Minidxr2 - Linux DVD player. 
 *
 * This file is from dvdplay, copyright (C) Creative Labs 1999
 * Changes in this version (C) Jack Whitham, 2001
 */
#include <config.h>
#ifdef HAVE_DXR2

#include <sys/types.h>
#include <dxr2ioctl.h>
#include <pthread.h>
#include <sys/ioctl.h>
#include <malloc.h>
#include <unistd.h>
#include <stdio.h>

#include "dxr2-api.h"
#include "output.h"
#include "setup.h"
#include "x11.h"

/* Translation between output states and dxr2 states */
output_xlat dxr2_output_xlat[] = { 
		{ OUTPUT_PLAYMODE_STOPPED , DXR2_PLAYMODE_STOPPED } ,
		{ OUTPUT_PLAYMODE_PAUSED , DXR2_PLAYMODE_PAUSED } ,
		{ OUTPUT_PLAYMODE_SLOWFORWARDS , DXR2_PLAYMODE_SLOWFORWARDS } ,
		{ OUTPUT_PLAYMODE_SLOWBACKWARDS , DXR2_PLAYMODE_SLOWBACKWARDS } ,
		{ OUTPUT_PLAYMODE_SINGLESTEP , DXR2_PLAYMODE_SINGLESTEP } ,
		{ OUTPUT_PLAYMODE_PLAY , DXR2_PLAYMODE_PLAY } ,
		{ OUTPUT_PLAYMODE_REVERSEPLAY , DXR2_PLAYMODE_REVERSEPLAY } ,
		{ OUTPUT_PLAYMODE_FASTFORWARDS , DXR2_PLAYMODE_FASTFORWARDS } ,
		{ OUTPUT_PLAYMODE_FASTBACKWARDS , DXR2_PLAYMODE_FASTBACKWARDS } ,
		{ OUTPUT_AUDIO_FREQ_441 , DXR2_AUDIO_FREQ_441 } ,
		{ OUTPUT_AUDIO_FREQ_48 , DXR2_AUDIO_FREQ_48 } ,
		{ OUTPUT_AUDIO_FREQ_96 , DXR2_AUDIO_FREQ_96 } ,
		{ OUTPUT_AUDIO_FREQ_2205 , DXR2_AUDIO_FREQ_2205 } ,
		{ OUTPUT_AUDIO_FREQ_32 , DXR2_AUDIO_FREQ_32 } ,
		{ OUTPUT_AUDIO_WIDTH_16 , DXR2_AUDIO_WIDTH_16 } ,
		{ OUTPUT_AUDIO_WIDTH_20 , DXR2_AUDIO_WIDTH_20 } ,
		{ OUTPUT_AUDIO_WIDTH_24 , DXR2_AUDIO_WIDTH_24 } ,
		{ OUTPUT_AUDIO_AC3 , DXR2_STREAM_AUDIO_AC3  } , 
		{ OUTPUT_AUDIO_MPEG , DXR2_STREAM_AUDIO_MPEG  } , 
		{ OUTPUT_AUDIO_LPCM , DXR2_STREAM_AUDIO_LPCM } , 
		{ OUTPUT_FORMAT_NTSC , DXR2_OUTPUTFORMAT_NTSC } , 
		{ OUTPUT_FORMAT_NTSC_60 , DXR2_OUTPUTFORMAT_NTSC_60 } , 
		{ OUTPUT_FORMAT_PAL_M , DXR2_OUTPUTFORMAT_PAL_M } , 
		{ OUTPUT_FORMAT_PAL_M_60 , DXR2_OUTPUTFORMAT_PAL_M_60 } , 
		{ OUTPUT_FORMAT_PAL_BDGHI , DXR2_OUTPUTFORMAT_PAL_BDGHI } , 
		{ OUTPUT_FORMAT_PAL_N , DXR2_OUTPUTFORMAT_PAL_N } , 
		{ OUTPUT_FORMAT_PAL_Nc , DXR2_OUTPUTFORMAT_PAL_Nc } , 
		{ OUTPUT_FORMAT_PAL_60 , DXR2_OUTPUTFORMAT_PAL_60 } , 
		{ OUTPUT_SRC_VIDEO_FREQ_30 , DXR2_SRC_VIDEO_FREQ_30 } , 
		{ OUTPUT_SRC_VIDEO_FREQ_25 , DXR2_SRC_VIDEO_FREQ_25 } , 
		{ OUTPUT_ASPECTRATIOMODE_LETTERBOX , DXR2_ASPECTRATIOMODE_LETTERBOX } , 
		{ OUTPUT_ASPECTRATIOMODE_NORMAL , DXR2_ASPECTRATIOMODE_NORMAL } , 
		{ OUTPUT_ASPECTRATIOMODE_PAN_SCAN , DXR2_ASPECTRATIOMODE_PAN_SCAN } , 
		{ OUTPUT_PLAYRATE_2x , DXR2_PLAYRATE_2x } ,
		{ OUTPUT_PLAYRATE_3x , DXR2_PLAYRATE_3x } ,
		{ OUTPUT_PLAYRATE_4x , DXR2_PLAYRATE_4x } ,
		{ OUTPUT_PLAYRATE_5x , DXR2_PLAYRATE_5x } ,
		{ OUTPUT_PLAYRATE_6x , DXR2_PLAYRATE_6x } ,
		{ OUTPUT_TYPE_VOB , DXR2_BITSTREAM_TYPE_MPEG_VOB } ,
		{ OUTPUT_TYPE_MPEG , DXR2_BITSTREAM_TYPE_MPEG_VCD } ,
		{ OUTPUT_END , 0 } } ;

static int bounds_check ( int num , int min , int max , int defaultval ) ;


int dxr2_init( dxr2_status_info_t * p )
{
	p -> dxr2_current_playmode = DXR2_PLAYMODE_STOPPED ;
	p -> overlay_initialised = 0 ;

	pthread_mutex_init ( & ( p -> view_mode_mutex ) , NULL ) ;
	p -> view_mode_setting = OUTPUT_VIEW_OFF ;

	return dxr2_reset( p );
}

int dxr2_initialise_overlay ( dxr2_status_info_t * p , output_extra_info * ck )
{
#ifdef HAVE_X_OVERLAY_SUPPORT
	int		rc ;

	/* It seems that this should really be called before doing ANYTHING
	 * else with the dxr2, which is a real pain. */

	if ( p -> dxr2_current_playmode != DXR2_PLAYMODE_STOPPED )
	{
		return 0 ;
	}
	p -> toplevel = NULL ;
	if ( ! x11_init_overlay_settings ( p ) )
	{
		return 0 ;
	}
	if ( ck != NULL )
	{
		p->color_key.arg1 = bounds_check ( ck -> red_min , 0 , 255 , 0x00 ) ;
		p->color_key.arg2 = bounds_check ( ck -> red_max , 0 , 255 , 0x40 ) ;
		p->color_key.arg3 = bounds_check ( ck -> green_min , 0 , 255 , 0x00 ) ;
		p->color_key.arg4 = bounds_check ( ck -> green_max , 0 , 255 , 0x40 ) ;
		p->color_key.arg5 = bounds_check ( ck -> blue_min , 0 , 255 , 0x60 ) ;
		p->color_key.arg6 = bounds_check ( ck -> blue_max , 0 , 255 , 0xff ) ;
		p->overlay_crop.arg1 = bounds_check ( ck -> left_crop , 0 , 1000 , 0 ) ;
		p->overlay_crop.arg2 = bounds_check ( ck -> right_crop , 0 , 1000 , 40 ) ;
		p->overlay_crop.arg3 = bounds_check ( ck -> top_crop , 0 , 1000 , 20 ) ;
		p->overlay_crop.arg4 = bounds_check ( ck -> bottom_crop , 0 , 1000 , 320 ) ;
	}
	(void) dxr2_reset ( p ) ;
	rc = x11_calculate_overlay_params ( p , 1 ) ; /* XXX */
	if ( rc )
	{
		p -> overlay_initialised = 1 ;
		(void) dxr2_reset ( p ) ;
	}
	return rc ;
#else
	return 0 ;
#endif
}

int dxr2_set_params( dxr2_status_info_t* dxr2_info)
{ 
  dxr2_select_video(dxr2_info, dxr2_info->video_stream.arg2);
  dxr2_set_video_freq(dxr2_info, dxr2_info->video_freq.arg1);
  dxr2_set_video_type(dxr2_info, dxr2_info->bitstream_type.arg);


  dxr2_set_tv_format(dxr2_info, dxr2_info->tv_format.arg);
  dxr2_set_output_ratio(dxr2_info, dxr2_info->output_aspect_ratio.arg);
  dxr2_set_source_ratio(dxr2_info, dxr2_info->source_aspect_ratio.arg);
  dxr2_set_scaling_mode(dxr2_info, dxr2_info->scaling_mode.arg);
  dxr2_set_macro_vision(dxr2_info, dxr2_info->macro_vision.arg);
  dxr2_set_pixel_mode(dxr2_info, dxr2_info->pixel_mode.arg);
  dxr2_set_interlace_mode(dxr2_info, dxr2_info->interlaced_mode.arg);

  // what are these?  Do we need a fn for them?
  ioctl(dxr2_info -> dxr2_fd, DXR2_IOC_SET_TV_75IRE_MODE, &dxr2_info->x75ire_mode);
  ioctl(dxr2_info -> dxr2_fd, DXR2_IOC_IEC958_OUTPUT_MODE, &dxr2_info->iec_output_mode);

  
  dxr2_subpicture(dxr2_info, dxr2_info->subpicture.arg);
  dxr2_select_subpicture(dxr2_info, dxr2_info->subpicture_stream_id.arg2);

  dxr2_select_audio(dxr2_info,  DXR2_STREAM_AUDIO_AC3 , 0 ) ;
  dxr2_set_audio_width(dxr2_info,  DXR2_AUDIO_WIDTH_16 ) ;
  dxr2_set_audio_freq(dxr2_info,  DXR2_AUDIO_FREQ_48 ) ;
  dxr2_set_volume(dxr2_info, dxr2_info->volume.arg);
  dxr2_mute(dxr2_info, dxr2_info->mute.arg);

  dxr2_set_overlay_gain(dxr2_info, dxr2_info->gain.arg1, dxr2_info->gain.arg2,
                       dxr2_info->gain.arg3, dxr2_info->gain.arg4);

  // should be setup before autodetection occurs
  dxr2_set_overlay_crop(dxr2_info, dxr2_info->overlay_crop.arg1, dxr2_info->overlay_crop.arg2,
                       dxr2_info->overlay_crop.arg3, dxr2_info->overlay_crop.arg4);
  

  // THIS does not appear to be implimented yet.
  //  dxr2_set_overlay_picture_controls(dxr2_info, dxr2_info->picture.arg1, dxr2_info->picture.arg2,
  //				    dxr2_info->picture.arg3, dxr2_info->picture.arg4);

  dxr2_set_in_delay(dxr2_info, dxr2_info->in_delay.arg);

  return 0 ;
}
 

int dxr2_set_up_audio( dxr2_status_info_t* dxr2_info)
{
  dxr2_set_audio_freq(dxr2_info, dxr2_info->audio_freq.arg);
  dxr2_set_audio_width(dxr2_info, dxr2_info->audio_width.arg);
  dxr2_select_audio(dxr2_info, dxr2_info->audio_stream.arg1, dxr2_info->audio_stream.arg2);
  return 1 ;
}

int dxr2_reset( dxr2_status_info_t * p )
{
  int rc ;

  rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_RESET , 0 );

  return rc ;
}

/* int dxr2_empty( dxr2_status_info_t * p )
{
  dxr2_oneArg_t buf;

  printf ( "in empty\n" ) ;
  while(1) {

    ioctl(p -> dxr2_fd, DXR2_IOC_BUFFERS_EMPTY, &buf);
	  printf ( "waiting...\n" ) ;
    if (buf.arg) {

      break;
    }

    usleep(100000);
  }
	  printf ( "exiting...\n" ) ;

  return 0 ;
} */


int dxr2_install_firmware( dxr2_status_info_t * p , dxr2_uCode_t* uCode)
{
  int rc ;

  if(uCode != NULL)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_INIT_ZIVADS, uCode);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_playmode( dxr2_status_info_t * p , int playmode)
{
  int rc ;
  int ioc=-1;

  if(playmode >= DXR2_PLAYMODE_STOPPED && playmode <= DXR2_PLAYMODE_FASTBACKWARDS && p -> dxr2_fd >= 0)
  {
    switch(playmode) {
    case DXR2_PLAYMODE_STOPPED:
      ioc = DXR2_IOC_STOP;
      break;
    case DXR2_PLAYMODE_PAUSED:
      ioc = DXR2_IOC_PAUSE;
      break;
    case DXR2_PLAYMODE_SLOWFORWARDS:
      ioc = DXR2_IOC_SLOW_FORWARDS;
      break;
    case DXR2_PLAYMODE_SLOWBACKWARDS:
      ioc = DXR2_IOC_SLOW_BACKWARDS;
      break;
    case DXR2_PLAYMODE_SINGLESTEP:
      ioc = DXR2_IOC_SINGLE_STEP;
      break;
    case DXR2_PLAYMODE_PLAY:
      ioc = DXR2_IOC_PLAY;
      break;
    case DXR2_PLAYMODE_REVERSEPLAY:
      ioc = DXR2_IOC_REVERSE_PLAY;
      break;
    case DXR2_PLAYMODE_FASTFORWARDS:
    case DXR2_PLAYMODE_FASTBACKWARDS:
      return -1;
	default :
      return -1;
    }
	p -> dxr2_current_playmode = playmode ;
  }

  if(ioc >= 0)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, ioc, NULL);
  else
    rc = -1;

  return rc ;
}

int dxr2_fast_forwards( dxr2_status_info_t * p , int rate)
{
  int rc ;

  dxr2_oneArg_t newRate;

  if(rate<DXR2_PLAYRATE_2x || rate>DXR2_PLAYRATE_6x)
    return -1;

  newRate.arg = rate;


  p -> dxr2_current_playmode = DXR2_PLAYMODE_FASTFORWARDS;

  rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_FAST_FORWARDS, &newRate);

  return rc ;
}

int dxr2_slow_forwards( dxr2_status_info_t * p , int rate)
{
  int rc ;

  dxr2_oneArg_t newRate;

  if(rate<DXR2_PLAYRATE_2x || rate>DXR2_PLAYRATE_6x)
    return -1;

  newRate.arg = rate;

  p -> dxr2_current_playmode = DXR2_PLAYMODE_SLOWFORWARDS;

  rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SLOW_FORWARDS, &newRate);

  return rc ;
}

int dxr2_set_tv_format( dxr2_status_info_t * p , int format)
{
  int rc ;

  if(format >= DXR2_OUTPUTFORMAT_NTSC && format <= DXR2_OUTPUTFORMAT_PAL_60)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_TV_OUTPUT_FORMAT, &format);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_color_key( dxr2_status_info_t * p , int red_min, int red_max, int green_min, int green_max,
			       int blue_min, int blue_max)
{
  int rc ;

  dxr2_sixArg_t colorKey;

  colorKey.arg1 = red_min;
  colorKey.arg2 = red_max;
  colorKey.arg3 = green_min;
  colorKey.arg4 = green_max;
  colorKey.arg5 = blue_min;
  colorKey.arg6 = blue_max;

  rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_COLOUR, &colorKey);

  return rc ;
}

int dxr2_set_output_ratio( dxr2_status_info_t * p , int ratio)
{
  int rc ;

  if(ratio >= DXR2_ASPECTRATIO_4_3 && ratio <= DXR2_ASPECTRATIO_16_9)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OUTPUT_ASPECT_RATIO, &ratio);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_source_ratio( dxr2_status_info_t * p , int ratio)
{
  int rc ;

  if(ratio >= DXR2_ASPECTRATIO_4_3 && ratio <= DXR2_ASPECTRATIO_16_9)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_SOURCE_ASPECT_RATIO, &ratio);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_scaling_mode( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode >= DXR2_ASPECTRATIOMODE_NORMAL && mode <= DXR2_ASPECTRATIOMODE_LETTERBOX)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_ASPECT_RATIO_MODE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_macro_vision( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode >= DXR2_MACROVISION_OFF && mode <= DXR2_MACROVISION_AGC_4COLOURSTRIPE)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_TV_MACROVISION_MODE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_pixel_mode( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode >= DXR2_PIXEL_CCIR601 && mode <= DXR2_PIXEL_SQUARE)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_TV_PIXEL_MODE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_interlace_mode( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode == DXR2_INTERLACED_OFF || mode == DXR2_INTERLACED_ON)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_TV_INTERLACED_MODE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_subpicture( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode == DXR2_SUBPICTURE_ON || mode==DXR2_SUBPICTURE_OFF)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_ENABLE_SUBPICTURE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_select_subpicture( dxr2_status_info_t * p , int stream)
{
  int rc ;
  dxr2_twoArg_t stream_id;

  stream_id.arg1 = DXR2_STREAM_SUBPICTURE;
  stream_id.arg2 = stream;

  if(stream >= 0 && stream <= 31)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SELECT_STREAM, &stream_id);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_subpicture_palette( dxr2_status_info_t * p , clut_t *pal )
{
  dxr2_palette_t dxr2_pal;
  int n;
  for (n=0; n<16; n++) {
    dxr2_pal.entries[n]=(pal[n].y << 16) + (pal[n].cr << 8) + (pal[n].cb);
  }
  return p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_SUBPICTURE_PALETTE, &dxr2_pal);
}

int dxr2_select_audio( dxr2_status_info_t * p , int type, int stream)
{
  int rc ;
  dxr2_threeArg_t stream_id;
  dxr2_oneArg_t	  ac3mode ;

  stream_id.arg1 = type;
  stream_id.arg2 = stream;

  if(stream >= 0 && stream <= 7)
  {
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SELECT_STREAM, &stream_id);
	if ( type == DXR2_STREAM_AUDIO_AC3 )
	{
		/* this may be better in it's own function. */
		ac3mode . arg = DXR2_AC3MODE_LR_STEREO_PROLOGIC ;
		rc = p -> dxr2_fd < 0 ? -1 : ioctl ( p -> dxr2_fd, DXR2_IOC_SET_AC3_MODE , & ac3mode ) ;
	}
  } else {
    rc = -1;
  }

  return rc ;
}

int dxr2_set_audio_width( dxr2_status_info_t * p , int width)
{
  int rc ;

  if(width >= DXR2_AUDIO_WIDTH_16 && width <= DXR2_AUDIO_WIDTH_24)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_AUDIO_DATA_WIDTH, &width);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_audio_freq( dxr2_status_info_t * p , int freq)
{
  int rc ;

  if(freq >= DXR2_AUDIO_FREQ_441 && freq <= DXR2_AUDIO_FREQ_32)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_AUDIO_SAMPLE_FREQUENCY, &freq);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_volume( dxr2_status_info_t * p , int volume)
{
  int rc ;

  if(volume >= 0 && volume <= 19)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_AUDIO_VOLUME, &volume);
  else
    rc = -1;

  return rc ;
}

int dxr2_mute( dxr2_status_info_t * p , int status)
{
  int rc ;

  if(status == DXR2_AUDIO_MUTE_ON || status == DXR2_AUDIO_MUTE_OFF)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_AUDIO_MUTE, &status);
  else
    rc = -1;

  return rc ;
}

int dxr2_select_video( dxr2_status_info_t * p , int stream)
{
  int rc ;

  dxr2_twoArg_t stream_id;

  stream_id.arg1 = DXR2_STREAM_VIDEO;
  stream_id.arg2 = stream;

  if(stream >= 0) // is there a max stream?
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SELECT_STREAM, &stream_id);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_video_freq( dxr2_status_info_t * p , int freq)
{
  int rc ;

  dxr2_threeArg_t video_freq;

  video_freq.arg1 = freq;
  video_freq.arg2 = 0x2d0; // what is this?
  video_freq.arg3 = 0x1e0; // what is this?

  if(freq == DXR2_SRC_VIDEO_FREQ_30 || freq == DXR2_SRC_VIDEO_FREQ_25)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_SOURCE_VIDEO_FORMAT, &video_freq);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_video_type( dxr2_status_info_t * p , int type)
{
  int rc ;

  if(type >= DXR2_BITSTREAM_TYPE_MPEG_VOB && type <= DXR2_BITSTREAM_TYPE_4)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_BITSTREAM_TYPE, &type);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_mode( dxr2_status_info_t * p , int mode)
{
  int rc ;

  if(mode >= DXR2_OVERLAY_DISABLED && mode <= DXR2_OVERLAY_WINDOW_COLOUR_KEY)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_MODE, &mode);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_ratio( dxr2_status_info_t * p , int ratio)
{
  int rc ;

  if(ratio >= 0 && ratio <= 2500) // is this the correct max?
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_RATIO, &ratio);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_geom( dxr2_status_info_t * p , geom_t geom)
{
  return dxr2_set_overlay_position(p, geom.x, geom.y) ||
         dxr2_set_overlay_size(p, geom.width, geom.height);
}

int dxr2_set_overlay_position( dxr2_status_info_t * p , int x, int y)
{
  int rc ;

  dxr2_twoArg_t pos;

  pos.arg1=x;
  pos.arg2=y;

  if(x >= 0 && y >= 0)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_POSITION, &pos);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_size( dxr2_status_info_t * p , int width, int height)
{
  int rc ;

  dxr2_twoArg_t size;

  size.arg1=width;
  size.arg2=height;

  if(width >= 0 && height >= 0)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_DIMENSION, &size);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_crop( dxr2_status_info_t * p , int left, int right, int top, int bottom)
{
  int rc ;

  dxr2_fourArg_t crop;
  
  crop.arg1=left;
  crop.arg2=right;
  crop.arg3=top;
  crop.arg4=bottom;

  if(left >= 0 && right>=0 && top>=0 && bottom>=0)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_CROPPING, &crop);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_in_delay( dxr2_status_info_t * p , int _delay)
{
  int rc ;

  dxr2_oneArg_t delay;

  delay.arg = _delay;
  if(_delay >= 0 && _delay <= 3)
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_IN_DELAY, &delay);
  else
    rc = -1;

  return rc ;
}

int dxr2_set_overlay_gain( dxr2_status_info_t * p , int common_gain, int red_gain, int green_gain, int blue_gain)
{
  int rc ;
  dxr2_fourArg_t picture;

  picture.arg1=common_gain;
  picture.arg2=red_gain;
  picture.arg3=green_gain;
  picture.arg4=blue_gain;

  if( common_gain >= 0 && common_gain <= 63 &&
      red_gain >= 0 && red_gain <= 63 &&
      green_gain >= 0 && green_gain <= 63 &&
      blue_gain >= 0 && blue_gain <= 63 )
  {
    rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_GAIN, &picture);
  } else {
	rc = -1;
  }

  return rc ;
}

int dxr2_set_overlay_picture_controls( dxr2_status_info_t * p , int gamma, int contrast, int brightness,
                                      int saturation)

{
	int rc ;
    dxr2_fourArg_t picture;
    
    picture.arg1 = gamma;
    picture.arg2 = contrast;
    picture.arg3 = brightness;
    picture.arg4 = saturation;
    
    if( gamma>=0 && gamma<= 96 &&
        contrast>=-128 && contrast<=127 &&
        brightness>=-128 && brightness<=127 &&
        saturation>=-128 && saturation<=127)
	{
        rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_SET_OVERLAY_PICTURE_CONTROLS, &picture);
	} else {
		rc = -1;
	}

	return rc ;
}

int dxr2_measure_vga_parameters( dxr2_status_info_t * p , dxr2_vgaParams_t* vgaParams)
{
  int rc ;

  rc = p -> dxr2_fd<0 ? -1 : ioctl(p -> dxr2_fd, DXR2_IOC_CALCULATE_VGA_PARAMETERS, vgaParams);

  return rc ;
}

 
int dxr2_setup_overlay( dxr2_status_info_t * p , dxr2_vgaParams_t* vgaParams )
{
    ioctl(p -> dxr2_fd, DXR2_IOC_SET_VGA_PARAMETERS, vgaParams);

    dxr2_set_overlay_color_key(p, p->color_key.arg1, p->color_key.arg2,
                               p->color_key.arg3, p->color_key.arg4,
                               p->color_key.arg5, p->color_key.arg6);
    dxr2_set_overlay_crop(p, p->overlay_crop.arg1, p->overlay_crop.arg2,
                          p->overlay_crop.arg3, p->overlay_crop.arg4);
    
    
    dxr2_set_overlay_size(p, p->overlay_dim.arg1, p->overlay_dim.arg2);
    dxr2_set_overlay_position(p, p->overlay_pos.arg1, p->overlay_pos.arg2);

    return dxr2_set_overlay_mode(p, p->overlay_mode.arg);
}

size_t dxr2_write ( dxr2_status_info_t * p , char * data , size_t size )
{
	size_t		rc ;

	rc = write ( p -> dxr2_fd , data , size ) ;

	return rc ;
}


int dxr2_ioctl ( dxr2_status_info_t * p , int ioctl_num , void * data )
{
	int		rc ;

    rc = ioctl(p -> dxr2_fd, ioctl_num , data ) ;

	return rc ;
}


/* Externalised functions.
 * These often just call the internal Dxr2 functions above.
 * The difference is that they use either simple boolean parameters/return codes
 * or the output codes defined in output.h. These codes are common to all output
 * devices, not just the Dxr2, and exist even if Dxr2 support is not being
 * compiled in.
 */

int dxr2_decryption_switch_ext ( dxr2_status_info_t * p , int css )
{
	dxr2_oneArg_t argBuf;

	argBuf.arg = css ? DXR2_CSSDECRMODE_ON : DXR2_CSSDECRMODE_OFF ; 
	if ( dxr2_ioctl ( p , DXR2_IOC_SET_DECRYPTION_MODE, &argBuf) ) 
	{
		return 0 ;
	} else {
		return 1 ;
	}
}

int dxr2_get_playmode_ext ( dxr2_status_info_t * p ) 
{
	return output_ext_to_int ( dxr2_output_xlat , p -> dxr2_current_playmode ) ;
}


int dxr2_fast_forwards_ext ( dxr2_status_info_t * p , int rate )
{
	return dxr2_fast_forwards ( p , output_int_to_ext ( dxr2_output_xlat , rate ) ) ;
}


int dxr2_mute_ext ( dxr2_status_info_t * p , int mute )
{
	return dxr2_mute ( p , mute ? DXR2_AUDIO_MUTE_ON : DXR2_AUDIO_MUTE_OFF ) ;
}

int dxr2_select_audio_ext ( dxr2_status_info_t * p , int type , int stream )
{
	return dxr2_select_audio ( p , output_int_to_ext ( dxr2_output_xlat , type ) ,
							output_int_to_ext ( dxr2_output_xlat , stream ) ) ;
}

int dxr2_set_audio_freq_ext ( dxr2_status_info_t * p , int freq )
{
	return dxr2_set_audio_freq ( p , output_int_to_ext ( dxr2_output_xlat , freq ) ) ;
}

int dxr2_set_audio_width_ext ( dxr2_status_info_t * p , int width )
{
	return dxr2_set_audio_width ( p , output_int_to_ext ( dxr2_output_xlat , width ) ) ;
}

int dxr2_set_playmode_ext ( dxr2_status_info_t * p , int pm )
{
	return dxr2_set_playmode ( p , output_int_to_ext ( dxr2_output_xlat , pm ) ) ;
}

int dxr2_subpicture_ext ( dxr2_status_info_t * p , int on )
{
	return dxr2_subpicture ( p , on ? DXR2_SUBPICTURE_ON : DXR2_SUBPICTURE_OFF ) ;
}

int dxr2_set_video_type_ext ( dxr2_status_info_t * p , int type )
{
	return dxr2_set_video_type ( p , output_int_to_ext ( dxr2_output_xlat , type ) ) ;
}

int dxr2_set_scaling_mode_ext ( dxr2_status_info_t * p , int mode )
{
	return dxr2_set_scaling_mode ( p , output_int_to_ext ( dxr2_output_xlat , mode ) ) ;
}

void dxr2_conditional_view_change ( dxr2_status_info_t * p , geom_t geom )
{
#ifdef HAVE_X_OVERLAY_SUPPORT
	/* for use by window functions in x11.c */
	pthread_mutex_lock ( & ( p -> view_mode_mutex ) ) ;
	if ( p -> view_mode_setting == OUTPUT_VIEW_WINDOW )
	{
		dxr2_set_overlay_color_key(p, p->color_key.arg1, p->color_key.arg2,
					   p->color_key.arg3, p->color_key.arg4,
					   p->color_key.arg5, p->color_key.arg6);
		dxr2_set_overlay_mode ( p , DXR2_OVERLAY_WINDOW_COLOUR_KEY ) ;
		dxr2_set_overlay_geom ( p , geom ) ;
	}
	pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
#endif
}

void dxr2_hide_overlay ( dxr2_status_info_t * p )
{
#ifdef HAVE_X_OVERLAY_SUPPORT
	geom_t		my_geom ;

	pthread_mutex_lock ( & ( p -> view_mode_mutex ) ) ;
	my_geom . x = my_geom . y = 0 ;
	my_geom . width = my_geom . height = 0 ;
	dxr2_set_overlay_color_key ( p ,
			255 , 255 , 255 , 255 , 255 , 255 ) ;
	dxr2_set_overlay_mode ( p , DXR2_OVERLAY_WINDOW_COLOUR_KEY ) ;
	dxr2_set_overlay_geom ( p , my_geom ) ;
	pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
#endif
}


int dxr2_set_view_mode ( dxr2_status_info_t * p , int mode , geom_t geom )
{
#ifdef HAVE_X_OVERLAY_SUPPORT
	geom_t		my_geom ;

	if (( ! p -> overlay_initialised )
	|| ( ! p -> overlay_avail ))
	{
		return ( mode == OUTPUT_VIEW_OFF ) ;
	}

	pthread_mutex_lock ( & ( p -> view_mode_mutex ) ) ;
	if (( mode == p -> view_mode_setting )
	&& ( mode != OUTPUT_VIEW_WINDOW ))
	{
		pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
		return 1 ;
	}
	switch ( mode )
	{
		case OUTPUT_VIEW_OFF :	/* Don't switch to DXR2_OVERLAY_DISABLED mode
								   once fullscreen has been on to work around
								   what appears to be a driver bug. */
								pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
								if ( p -> window_open )
								{
									x11_close_window ( p ) ;
								} else {
									dxr2_hide_overlay ( p ) ;
								}
								pthread_mutex_lock ( & ( p -> view_mode_mutex ) ) ;
								break ;
		case OUTPUT_VIEW_FULLSCREEN :
								my_geom . width = p -> vgaBuf . xScreen + 10 ;
								my_geom . height = p -> vgaBuf . yScreen ;
								my_geom . x = my_geom . y = 0 ;
								dxr2_set_overlay_mode ( p , DXR2_OVERLAY_WINDOW_KEY ) ;
								dxr2_set_overlay_geom ( p , my_geom ) ;
								break ;
		case OUTPUT_VIEW_WINDOW :
								pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
								if ( ! p -> window_open )
								{
									geom_t	default_geom ;

									default_geom . x = 100 ;
									default_geom . y = 100 ;
									default_geom . width = 400 ;
									default_geom . height = 300 ;

									x11_open_window ( p , default_geom ) ;
								} else {
									/* force a window refresh */
									x11_window_refresh ( p ) ;
								}
								pthread_mutex_lock ( & ( p -> view_mode_mutex ) ) ;
								break ;
		default : 				pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
								return 0 ;
	}
	p -> view_mode_setting = mode ;
	pthread_mutex_unlock ( & ( p -> view_mode_mutex ) ) ;
	return 1 ;
#else
	return ( mode == OUTPUT_VIEW_OFF ) ;
#endif
}

static int bounds_check ( int num , int min , int max , int defaultval )
{
	if (( num < min )
	|| ( num > max ))
	{
		return defaultval ;
	} else {
		return num ;
	}
}


#endif

