#include <config.h>
#ifdef HAVE_DXR2

#ifndef DXR2_API_H
#define DXR2_API_H

#ifdef HAVE_X_OVERLAY_SUPPORT
#include <X11/Intrinsic.h>
#endif

#include <dxr2ioctl.h>
#include <pthread.h>
#include "player.h"
#include "output.h"

typedef struct {
  dxr2_oneArg_t    tv_format;
  dxr2_threeArg_t  video_freq;
  dxr2_oneArg_t    output_aspect_ratio;
  dxr2_oneArg_t    source_aspect_ratio;
  dxr2_oneArg_t    scaling_mode;
  dxr2_oneArg_t    bitstream_type;
  dxr2_oneArg_t    macro_vision;
  dxr2_oneArg_t    interlaced_mode;
  dxr2_oneArg_t    pixel_mode;
  dxr2_oneArg_t    x75ire_mode;
  dxr2_oneArg_t    volume;
  dxr2_oneArg_t    mute;
  dxr2_oneArg_t    audio_width;
  dxr2_oneArg_t    audio_freq;
  dxr2_oneArg_t    iec_output_mode;  // not sure what this is
  dxr2_oneArg_t    subpicture;
  dxr2_twoArg_t    subpicture_stream_id;
  dxr2_twoArg_t    audio_stream;
  dxr2_twoArg_t    video_stream;
  dxr2_fourArg_t   overlay_crop;
  dxr2_twoArg_t    overlay_pos;
  dxr2_twoArg_t    overlay_dim;
  dxr2_oneArg_t    overlay_mode;
  dxr2_oneArg_t    in_delay;
  dxr2_fourArg_t   picture;
  dxr2_fourArg_t   gain;
  dxr2_vgaParams_t vgaBuf;
  dxr2_sixArg_t    color_key;
  int			   dxr2_fd ;
  int			   dxr2_current_playmode ;
  int			   overlay_avail ;
  int			   overlay_initialised ;
  int			   view_mode_setting ;
  pthread_mutex_t  view_mode_mutex ;
#ifdef HAVE_X_OVERLAY_SUPPORT
  Widget			toplevel ;
  int				window_open ;
  pthread_t			x_thread ;
  pthread_mutex_t	x_thread_mutex ;
  XtAppContext		application ;
  int				quit_flag , win_refresh_flag ;
  int				max_x , max_y ;
#endif
} dxr2_status_info_t;


extern int dxr2_init( dxr2_status_info_t * p );
extern int dxr2_set_params( dxr2_status_info_t*);
extern int dxr2_set_up_audio( dxr2_status_info_t* dxr2_info) ;
extern int dxr2_reset( dxr2_status_info_t * p );
/* extern int dxr2_empty( dxr2_status_info_t * p ); */

extern int dxr2_install_firmware( dxr2_status_info_t * p , dxr2_uCode_t*);

extern int dxr2_set_playmode( dxr2_status_info_t * p , int);
extern int dxr2_fast_fowards( dxr2_status_info_t * p , int);
extern int dxr2_slow_forwards( dxr2_status_info_t * p , int);

extern int dxr2_set_tv_format( dxr2_status_info_t * p , int);
extern int dxr2_set_output_ratio( dxr2_status_info_t * p , int);
extern int dxr2_set_source_ratio( dxr2_status_info_t * p , int);
extern int dxr2_set_scaling_mode( dxr2_status_info_t * p , int);
extern int dxr2_set_macro_vision( dxr2_status_info_t * p , int);
extern int dxr2_set_pixel_mode( dxr2_status_info_t * p , int);
extern int dxr2_set_interlace_mode( dxr2_status_info_t * p , int);

extern int dxr2_subpicture( dxr2_status_info_t * p , int);
extern int dxr2_select_subpicture( dxr2_status_info_t * p , int);
extern int dxr2_set_subpicture_palette( dxr2_status_info_t * p , clut_t *pal);

extern int dxr2_select_audio( dxr2_status_info_t * p , int, int);
extern int dxr2_set_audio_width( dxr2_status_info_t * p , int);
extern int dxr2_set_audio_freq( dxr2_status_info_t * p , int);
extern int dxr2_set_volume( dxr2_status_info_t * p , int);
extern int dxr2_mute( dxr2_status_info_t * p , int);

extern int dxr2_select_video( dxr2_status_info_t * p , int);
extern int dxr2_set_video_freq( dxr2_status_info_t * p , int);
extern int dxr2_set_video_type( dxr2_status_info_t * p , int);

extern int dxr2_set_overlay_ratio( dxr2_status_info_t * p , int);
extern int dxr2_set_overlay_geom( dxr2_status_info_t * p , geom_t);
extern int dxr2_set_overlay_position( dxr2_status_info_t * p , int, int);
extern int dxr2_set_overlay_size( dxr2_status_info_t * p , int, int);
extern int dxr2_set_overlay_crop( dxr2_status_info_t * p , int, int, int, int);
extern int dxr2_set_in_delay( dxr2_status_info_t * p , int);
extern int dxr2_set_overlay_gain( dxr2_status_info_t * p , int, int, int, int);
extern int dxr2_set_overlay_picture_controls( dxr2_status_info_t * p , int, int, int, int);
extern int dxr2_set_overlay_color_key( dxr2_status_info_t * p , int, int, int, int, int, int);

extern int dxr2_fast_forwards( dxr2_status_info_t * p , int rate) ;
extern int dxr2_set_overlay_mode( dxr2_status_info_t * p , int mode) ;
extern int dxr2_setup_overlay( dxr2_status_info_t * p , dxr2_vgaParams_t* vgaParams ) ;

extern size_t dxr2_write ( dxr2_status_info_t * p , char * data , size_t size ) ;

extern int dxr2_ioctl ( dxr2_status_info_t * p , int ioctl_num , void * data ) ;

extern int dxr2_decryption_switch_ext ( dxr2_status_info_t * p , int css ) ;
extern int dxr2_get_playmode_ext ( dxr2_status_info_t * p ) ;
extern int dxr2_fast_forwards_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_select_audio_ext ( dxr2_status_info_t * p , int , int ) ;
extern int dxr2_set_audio_freq_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_set_audio_width_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_set_playmode_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_subpicture_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_mute_ext ( dxr2_status_info_t * p , int ) ;
extern int dxr2_set_video_type_ext ( dxr2_status_info_t * p , int type ) ;
extern int dxr2_set_scaling_mode_ext ( dxr2_status_info_t * p , int mode ) ;

extern int dxr2_set_view_mode ( dxr2_status_info_t * p , int mode , geom_t geom ) ;
extern int dxr2_initialise_overlay ( dxr2_status_info_t * p , output_extra_info * ck ) ;
extern void dxr2_conditional_view_change ( dxr2_status_info_t * p , geom_t geom ) ;
extern void dxr2_hide_overlay ( dxr2_status_info_t * p ) ;

extern output_xlat dxr2_output_xlat[] ;

#endif


#endif

