
/*
 * Based on input_udf.c -
 * Copyright (C) 1999-2000  Thomas Mirlacher
 *
 * Changes copyright (C) 2001 Jack Whitham
 *
 * This is an interface to the libcss decoding library.
 * No actual DeCSS code is contained within this program,
 * but libcss can be obtained from the LiViD project at:-
 *
 * http://www.linuxvideo.org/
 *
 * This code will automatically use libcss if it is
 * available for decoding DVDs.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <errno.h>
#include <sys/ioctl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <dlfcn.h>

#include <player.h>
#include <output.h>
#include <config.h>

#include "interface.h"



typedef struct file_status_struct {
				int		css_not_encrypted ;
				char	key_disc [ CHUNK_SIZE ] ;
				char	key_title [ 5 ] ;
				int		out_fd ;
				int		play_mode ;
				int 	(* CSSisEncrypted_ptr) ( int fd ) ;
				int 	(* CSSAuthDisc_ptr) ( int , char * key_disc ) ;
				int 	(* CSSAuthTitle_ptr) ( int , char * key_title , int lba ) ;
				int 	(* CSSGetASF_ptr) ( int ) ;
				int 	(* CSSDecryptTitleKey_ptr) ( char * key_title , char * key_disc ) ;
				void 	(* CSSDescramble_ptr) ( u_char * sec , u_char * key ) ;
				void	* dl_handle ;
		} file_status ;
				

static int soft_decryption_switch ( file_status * state , int css )
{
	if ( ! css )
	{
		state -> css_not_encrypted = 1 ;
	}
	return 1 ;
}

static int soft_write ( file_status * state , char * data , int length )
{
	if ( state -> dl_handle != NULL )
	{
		if ( ! state -> css_not_encrypted )
		{
			int			n , seglen ;

			for ( n = 0 ; n < length ; n += CHUNK_SIZE )
			{
				seglen = length - n ;
				if (( seglen < CHUNK_SIZE )
				&& ( seglen > 0 ))
				{
					char		data_copy [ CHUNK_SIZE ] ;

					memset ( data_copy , '\0' , CHUNK_SIZE ) ;
					memcpy ( data_copy , & data [ n ] , seglen ) ;
					(* state -> CSSDescramble_ptr) ( data_copy , state -> key_title ) ;
					memcpy ( & data [ n ] , data_copy , seglen ) ;
				} else if ( seglen >= CHUNK_SIZE )
				{
					(* state -> CSSDescramble_ptr) ( & data [ n ] , state -> key_title ) ;
				}
			}
		}
	}

	if ( state -> play_mode != OUTPUT_PLAYMODE_PLAY )
	{
		return length ;
	} else {
		return write ( state -> out_fd , data , length ) ;
	}
}


static int soft_css_authenticate ( css_manager * css , file_status * state , 
			char * dvd_device , char * error , int max_chars )
{
	if ( css -> css_ready )
	{
		SAFE_STRNCPY ( error , "CSS: already set up.\n" , max_chars ) ;
		return OUTPUT_CSS_ERROR ;
	}

	if ( state -> dl_handle != NULL )
	{
		css -> css_ready = 1 ;

		if ( strlen ( dvd_device ) > 0 )
		{
			css -> drive_fd = open ( dvd_device , O_RDONLY | O_NONBLOCK ) ;
			if ( css -> drive_fd < 0 )
			{
				SAFE_STRNCPY ( error , "CSS: cannot open dvd device.\n" , max_chars ) ;
				css -> css_ready = 0 ;
				return OUTPUT_CSS_ERROR ;
			}
		} else {
			SAFE_STRNCPY ( error , "CSS: no dvd device named. Check config file.\n" , max_chars ) ;
			css -> css_ready = 0 ;
			return OUTPUT_CSS_ERROR ;
		}

		return OUTPUT_CSS_OK ;
	} else {
		SAFE_STRNCPY ( error , "This title is encrypted, and libcss is not available.\n" , max_chars ) ;
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}
}

static int soft_css_do_disc_key ( css_manager * css ,
			file_status * state , char * error , int max_chars )
{
	if ( ! css -> css_ready )
	{
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}
	if ( state -> dl_handle != NULL )
	{
		css -> css_not_encrypted = state -> css_not_encrypted = 0 ;
		if ( ! (* state -> CSSisEncrypted_ptr) ( css -> drive_fd ) )
		{
			css -> css_not_encrypted = state -> css_not_encrypted = 1 ;
			return OUTPUT_CSS_NOT_ENCRYPTED ;
		}
		if ( (* state -> CSSAuthDisc_ptr) ( css -> drive_fd , state -> key_disc ) < 0 )
		{
			SAFE_STRNCPY ( error , "CSSAuthDisc\n" , max_chars ) ;
					/* strerror ( errno ) ) ; XXX */
			return OUTPUT_CSS_ERROR ;
		} else {
			return OUTPUT_CSS_OK ;
		}
	} else {
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}
}

static int soft_css_do_title_key ( css_manager * css , 
			file_status * state , char * error , int max_chars , int lba )
{
	if ( ! css -> css_ready )
	{
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}

	if ( state -> dl_handle != NULL )
	{
		if ( ! (* state -> CSSisEncrypted_ptr) ( css -> drive_fd ) )
		{
			return OUTPUT_CSS_NOT_ENCRYPTED ;
		}
		if ( (* state -> CSSAuthTitle_ptr) ( css -> drive_fd , state -> key_title , lba ) < 0 )
		{
			SAFE_STRNCPY ( error , "CSSAuthTitle\n" , max_chars ) ;
					/* strerror ( errno ) ) ; XXX */
			return OUTPUT_CSS_ERROR ;
		}
		if ( (* state -> CSSDecryptTitleKey_ptr) ( state -> key_title , state -> key_disc ) < 0 )
		{
			SAFE_STRNCPY ( error , "CSSDecryptTitleKey (%s)\n" , max_chars ) ;
					/* strerror ( errno ) ) ; XXX */
			return OUTPUT_CSS_ERROR ;
		}
		return OUTPUT_CSS_OK ;
	} else {
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}
}

static int soft_css_finish ( css_manager * css )
{
	int				i ;
	dvd_authinfo	ai ;

	if ( ! css -> css_ready )
	{
		return OUTPUT_CSS_NOT_AVAILABLE ;
	}

	css -> css_ready = 0 ;

/* reset all AGIDs on the DVD drive */
	for (i=0; i<4; i++) 
	{
		memset (&ai,  0, sizeof (dvd_authinfo));
		ai.type = DVD_INVALIDATE_AGID;
		ai.lsa.agid = i;
		ioctl ( css -> drive_fd , DVD_AUTH, &ai);
	}

	close ( css -> drive_fd ) ;

	return OUTPUT_CSS_OK ;
}

static char * soft_css_reset ( char * dvd_device )
{
	dvd_authinfo ai;
	int i;
	int fd ;

	fd = open ( dvd_device, O_RDONLY | O_NONBLOCK);

	if (fd < 0) {
		return "unable to open dvd drive\n" ;
	}

	for (i = 0; i < 4; i++) 
	{
		memset(&ai, 0, sizeof(ai));
		ai.type = DVD_INVALIDATE_AGID;
		ai.lsa.agid = i;
		ioctl(fd, DVD_AUTH, &ai);
	}

	close ( fd ) ;

	return NULL ;
}

static file_status * soft_css_initialise 
		( player_info_t * player_info , char ** error )
{
	file_status		* fs ;

	if ( player_info -> out_file == NULL )
	{
		(* error) = "No output file specified for file output mode.\n" ;
		return NULL ;
	}

	fs = calloc ( 1 , sizeof ( file_status ) ) ;

	fs -> play_mode = OUTPUT_PLAYMODE_STOPPED ;
	fs -> out_fd = open ( player_info -> out_file , O_WRONLY | O_CREAT ,
			S_IRUSR | S_IRGRP | S_IROTH | S_IWUSR ) ;

	if ( fs -> out_fd < 0 )
	{
		(* error) = "Unable to open/create specified file.\n" ;
		free ( fs ) ;
		return NULL ;
	}

	fs -> dl_handle = dlopen ( "libcss.so" , RTLD_LAZY ) ;
	if ( fs -> dl_handle != NULL )
	{
		if ((( fs -> CSSisEncrypted_ptr = dlsym ( fs -> dl_handle , "CSSisEncrypted" )) != NULL )
		&& (( fs -> CSSAuthDisc_ptr = dlsym ( fs -> dl_handle , "CSSAuthDisc" )) != NULL )
		&& (( fs -> CSSAuthTitle_ptr = dlsym ( fs -> dl_handle , "CSSAuthTitle" )) != NULL )
		&& (( fs -> CSSGetASF_ptr = dlsym ( fs -> dl_handle , "CSSGetASF" )) != NULL )
		&& (( fs -> CSSDecryptTitleKey_ptr = dlsym ( fs -> dl_handle , "CSSDecryptTitleKey" )) != NULL )
		&& (( fs -> CSSDescramble_ptr = dlsym ( fs -> dl_handle , "CSSDescramble" )) != NULL ))
		{
			/* libcss successfully loaded. */
		} else {
			dlclose ( fs -> dl_handle ) ;
			fs -> dl_handle = NULL ;
		}
	}
	return fs ;
}

static char * soft_css_deinitialise ( file_status * state )
{
	if ( state -> dl_handle != NULL )
	{
		dlclose ( state -> dl_handle ) ;
	}
	close ( state -> out_fd ) ;

	free ( state ) ;

	return NULL ;
}

static int soft_set_playmode ( file_status * state , int pm )
{
	state -> play_mode = pm ;
	return 1 ;
}

static int soft_get_playmode ( file_status * state )
{
	return state -> play_mode ;
}


/*
 * Get the output plugin definition for the file/pipe output subsystem
 *
 */

int file_plugin ( output_plugin * o )
{
	o -> decryption_switch = (output_control_fn_1_args) soft_decryption_switch ;
	o -> get_playmode = (output_control_fn_0_args) soft_get_playmode ;
	o -> do_write = (output_write_fn) soft_write ;
	o -> do_css_finish = (output_css_finish_fn) soft_css_finish ;
	o -> do_css_authenticate = (output_css_auth_fn) soft_css_authenticate ;
	o -> do_css_do_disc_key = (output_css_disc_fn) soft_css_do_disc_key ;
	o -> do_css_do_title_key = (output_css_title_fn) soft_css_do_title_key ;
	o -> do_css_reset = (output_css_reset_fn) soft_css_reset ;
	o -> set_playmode = (output_control_fn_1_args) soft_set_playmode ;
	o -> do_initialise = (output_initialise_fn) soft_css_initialise ;
	o -> do_deinitialise = (output_deinitialise_fn) soft_css_deinitialise ; 

	return 1 ;
}

