/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
#ifndef IFO_PARSE_H
#define IFO_PARSE_H

#include "player.h"

/* IFO return codes */
#define IFO_NOT_AVAILABLE		-7
#define IFO_SILLY_PARAMETERS	-6
#define IFO_UNKNOWN_TITLE		-5
#define IFO_NO_TABLE			-4
#define IFO_SILLY_DATA			-3
#define IFO_OFF_END_OF_FILE		-2
#define IFO_UNKNOWN				-1
#define IFO_OK					0
#define IFO_CONFUSED			1
#define IFO_END_OF_TITLE		2

#define IFO_DFMODE_FIND_END 		1
#define IFO_DFMODE_FIND_NEXT		2
#define IFO_DFMODE_FIND_BEGINNING	3

#define IFO_LANGCODE_NORMAL		3
#define IFO_LANGCODE_COMMENTARY	4
#define IFO_LANGCODE_UNKNOWN	5

/* IFO access functions */
int ifo_get_time_for_position ( player_info_t * p , int title , int position ,
		int * hour , int * min , int * second ) ;
int ifo_direct_get_time_for_position ( ifo_t * the_ifo , int position , 
		int * hour , int * min , int * second ) ;
int ifo_get_position_for_time ( player_info_t * p , int title , 
		int hour , int min , int second ) ;
int ifo_settings_audio_width ( player_info_t * p , int title , int stream ) ;
int ifo_settings_audio_freq ( player_info_t * p , int title , int stream ) ;
int ifo_settings_audio_format ( player_info_t * p , int title , int stream ) ;
int ifo_settings_pan_scan ( player_info_t * p , int title ) ;
int ifo_settings_aspect_16_9 ( player_info_t * p , int title ) ;
int ifo_get_num_audio_streams ( player_info_t * p , int title ) ;
int ifo_settings_langcode ( player_info_t * p , int title , int stream ) ;
int ifo_get_num_sub_streams ( player_info_t * p , int title ) ;
int ifo_settings_sub_langcode ( player_info_t * p , int title , int stream ,
					int * langcode ) ;
int ifo_settings_num_channels ( player_info_t * p , int title , int stream ) ;
int ifo_get_pos_from_main_title_search_pointer ( player_info_t * p , int map_number ,
			int * title , int * angle ) ;
int ifo_compute_playback_stack ( player_info_t * p , int title ) ;
void ifo_free_playback_stack ( player_info_t * p ) ;
int ifo_data_follower ( player_info_t * p , int * position , int * until , 
				int * title , int desired_angle , int find_next ) ;
int ifo_available ( player_info_t * player_info ) ;

int ifo_open ( player_info_t * player_info ) ;
int ifo_close ( player_info_t * p ) ;

void ifo_setup ( player_info_t * player_info ) ;

void ifo_recompute_tables ( player_info_t * p ) ;
int ifo_get_chapter_position ( player_info_t * p , int title , int chapter ) ;
int ifo_get_num_chapters ( player_info_t * p , int title ) ;

ifo_t * ifo_examine_title ( player_info_t * player_info , int title , int * fd ) ;

#endif

