/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
#include <string.h>
#include <pthread.h>

#include "player.h"
#include "queue.h"

#define MAX_QUEUE_SIZE		128


typedef struct message_struct {
				int		code ;
				char	* data ;
			} message ;

int msg_send_message ( player_info_t * p , int code , char * data )
{
	message		* msg ;
	int			rc ;

	msg = calloc ( 1 , sizeof ( message ) ) ;
	msg -> code = code ;
	if ( data == NULL )
	{
		msg -> data = NULL ;
	} else {
		msg -> data = strdup ( data ) ;
	}
	pthread_mutex_lock ( & ( p -> message_queue_mutex ) ) ;
	rc = queue_add_to_queue ( p -> message_queue , msg ) ;
	pthread_mutex_unlock ( & ( p -> message_queue_mutex ) ) ;
	if ( ! rc )
	{
		if ( msg -> data != NULL )
		{
			free ( msg -> data ) ;
		}
		free ( msg ) ;
	}
	return rc ;
}

int msg_get_message ( player_info_t * p , int * code , char ** data )
{
	message		* msg ;

	pthread_mutex_lock ( & ( p -> message_queue_mutex ) ) ;
	msg = (message *) queue_remove_from_queue ( p -> message_queue ) ;
	pthread_mutex_unlock ( & ( p -> message_queue_mutex ) ) ;

	if ( msg == NULL )
	{
		return 0 ;	/* no messages */
	} else {
		(* code) = msg -> code ;
		(* data) = msg -> data ;
		free ( msg ) ;
		return 1 ;
	}
}

void msg_setup ( player_info_t * p )
{
	pthread_mutex_init ( & ( p -> message_queue_mutex ) , NULL ) ;
	p -> message_queue = queue_make_queue ( MAX_QUEUE_SIZE ) ;
}

