/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
#ifndef OUTPUT_H
#define OUTPUT_H

#include <pthread.h>
#include <sys/types.h>
#include <linux/cdrom.h>

#include "output_codes.h"
#include "ifo/libifo/ifo.h"

#define NUM_BITRATE_RECORDS					3

typedef struct {
				  int width;
				  int height;
				  int x;
				  int y;
				} geom_t;

typedef struct output_extra_info_struct {
				int red_min ;
				int red_max ;
				int green_min ;
				int green_max ;
				int blue_min ;
				int blue_max ;
				int left_crop ;
				int right_crop ;
				int top_crop ;
				int bottom_crop ;
				} output_extra_info ;

typedef int (* output_plugin_fn) ( void * o ) ;
typedef int (* output_control_fn_0_args) ( void * ) ;
typedef int (* output_control_fn_1_args) ( void * , int ) ;
typedef int (* output_control_fn_2_args) ( void * , int , int ) ;
typedef int (* output_css_finish_fn) ( void * ) ;
typedef int (* output_css_disc_fn) ( void * , void * , char * , int ) ;
typedef int (* output_css_title_fn) ( void * , void * , char * , int , int ) ;
typedef int (* output_css_auth_fn) ( void * , void * , char * , char * , int ) ;
typedef char * (* output_css_reset_fn) ( char * ) ;
typedef size_t (* output_write_fn) ( void * , char * , size_t ) ;
typedef int (* output_ioctl_fn) ( void * , int , void * ) ;
typedef int (* output_set_view_mode_fn) ( void * , int , geom_t ) ;
typedef int (* output_initialise_overlay_fn) ( void * , output_extra_info * ) ;
typedef void * (* output_initialise_fn) ( void * , char ** ) ;
typedef char * (* output_deinitialise_fn) ( void * ) ;

typedef struct css_manager_struct {
				dvd_authinfo	auth ;
				int				css_ready ;
				int				drive_fd ;
				int				css_not_encrypted ;
				} css_manager ;
		
typedef struct output_plugin_struct {
				/* the plugin function should not initialise these variables */
				void 							* state_data ;
				pthread_mutex_t					* output_mutex ;
				css_manager						* css ;
				time_t							bitrate_records_start ;
				int								bitrate_records_total [ NUM_BITRATE_RECORDS ] ;


				/* general settings */
				output_control_fn_1_args		decryption_switch ;
				output_control_fn_1_args		fast_forwards ;
				output_control_fn_0_args		get_playmode ;
				output_control_fn_1_args		mute ;
				output_control_fn_0_args		reset ;
				output_control_fn_2_args		select_audio ;
				output_control_fn_1_args		select_subpicture ;
				output_control_fn_1_args		set_audio_freq ;
				output_control_fn_1_args		set_audio_width ;
				output_control_fn_1_args		set_playmode ;
				output_control_fn_1_args		subpicture ;
				output_control_fn_1_args		set_video_type ;
				output_control_fn_1_args		set_scaling_mode ;

				/* special controls */
				output_write_fn					do_write ;
				output_ioctl_fn					do_ioctl ;
				output_initialise_fn			do_initialise ;
				output_deinitialise_fn			do_deinitialise ;

				/* css controls */
				output_css_finish_fn			do_css_finish ;
				output_css_auth_fn				do_css_authenticate ;
				output_css_disc_fn				do_css_do_disc_key ;
				output_css_title_fn				do_css_do_title_key ;
				output_css_reset_fn				do_css_reset ;
				
				/* fullscreen etc */
				output_set_view_mode_fn			do_set_view_mode ;
				output_initialise_overlay_fn	do_initialise_overlay ;

                output_control_fn_1_args        set_subpicture_palette ;
			} output_plugin ;

typedef struct output_xlat_struct {
				int internal_value ;
				int external_value ;
			} output_xlat ;


output_plugin * output_setup ( void * p , 
			output_plugin_fn out_fn , char ** err ) ;
char * output_deinitialise ( output_plugin * o ) ;
int output_decryption_switch ( output_plugin * o , int param ) ;
int output_fast_forwards ( output_plugin * o , int param ) ;
int output_get_playmode ( output_plugin * o ) ;
int output_mute ( output_plugin * o , int param ) ;
int output_reset ( output_plugin * o ) ;
int output_select_audio ( output_plugin * o , int param1 , int param2 ) ;
int output_select_subpicture ( output_plugin * o , int param ) ;
int output_set_subpicture_palette ( output_plugin * o , clut_t * pal ) ;
int output_set_audio_freq ( output_plugin * o , int param ) ;
int output_set_audio_width ( output_plugin * o , int param ) ;
int output_set_playmode ( output_plugin * o , int param ) ;
int output_subpicture ( output_plugin * o , int param ) ;
int output_write ( output_plugin * o , char * data , int param ) ;
int output_ioctl ( output_plugin * o , int num , void * data ) ;

int output_css_authenticate ( output_plugin * o , char * dvd_device ,
					char * err , int max_chars ) ;
int output_css_finish ( output_plugin * o ) ;
int output_css_do_disc_key ( output_plugin * o , char * err , int max_chars ) ;
int output_css_do_title_key ( output_plugin * o , char * err , int max_chars , int lba ) ;
char * output_css_do_reset ( output_plugin * o , char * dvd_device ) ;

int output_int_to_ext ( output_xlat * xlat , int internal ) ;
int output_ext_to_int ( output_xlat * xlat , int external ) ;
void output_if_valid_int_then_set ( output_xlat * xlat , int * set_me , int int_value ) ;
int output_set_video_type ( output_plugin * o , int type ) ;
int output_set_scaling_mode ( output_plugin * o , int mode ) ;
int output_set_view_mode ( output_plugin * o , int mode , geom_t geom ) ;
int output_initialise_overlay ( output_plugin * o , output_extra_info * ck ) ;

int output_get_bitrate ( output_plugin * o ) ;

#endif

