/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
#include <string.h>
#include <stdio.h>
#include <ctype.h>

#include "parser.h"
#include "player.h"

int parse_type ( char * input , parser_table * t ,
		void ** output_data , void ** output_data_2 , char * name , char * error_buffer )
{
	int			i ;

	if ( error_buffer != NULL )
	{
		error_buffer [ 0 ] = '\0' ;
	}
	for ( i = 0 ; strlen ( t [ i ] . key ) > 0 ; i ++ )
	{
		if ( strcasecmp ( input , t [ i ] . key ) == 0 )
		{
			if ( output_data != NULL )
			{
				(* output_data) = t [ i ] . data ;
			}
			if ( output_data_2 != NULL )
			{
				(* output_data_2) = t [ i ] . data2 ;
			}
			return t [ i ] . type ;		/* success */
		}
	}

	if (( error_buffer != NULL )
	&& ( name != NULL ))
	{
		snprintf ( error_buffer , SHORT_BUFFER_LENGTH_1 , "'%s' is an unrecognised %s\n" ,
				input , name ) ;
	}
	return 0 ;
}

int parse_reverse ( int input , parser_table * t ,
		char ** output_data , char * name , char * error_buffer )
{
	int			i ;

	if ( error_buffer != NULL )
	{
		error_buffer [ 0 ] = '\0' ;
	}
	(* output_data) = NULL ;

	for ( i = 0 ; strlen ( t [ i ] . key ) > 0 ; i ++ )
	{
		if ( input == ( (int) t [ i ] . data ) )
		{
			(* output_data) = t [ i ] . key ;
			return t [ i ] . type ;
		}
	}

	if (( error_buffer != NULL )
	&& ( name != NULL ))
	{
		snprintf ( error_buffer , SHORT_BUFFER_LENGTH_1 , "Value '%d' is an unrecognised %s\n" ,
				input , name ) ;
	}
	return 0 ;
}


void remove_whitespace ( char * str )
{
	int			i ;

	if ( strlen ( str ) > 0 )
	{
		/* leading.. */
		for ( i = 0 ; isspace ( str [ i ] ) ; i ++ )
		{
		}

		if ( i > 0 )
		{
			memmove ( str , & str [ i ] , strlen ( & str [ i ] ) + 1 ) ;
		}

		/* trailing */
		for ( i = strlen ( str ) - 1 ; isspace ( str [ i ] ) && ( i >= 0 ) ; i -- )
		{
			str [ i ] = '\0' ;
		}
	}
}

char * find_index_of_first ( char * string , char * items )
{
	int			i ;
	char		* firstpos = NULL ;
	char		* pos ;

	for ( i = 0 ; i < strlen ( items ) ; i ++ )
	{
		pos = index ( string , items [ i ] ) ;
		if (( pos != NULL )
		&& (( pos < firstpos )
		|| ( firstpos == NULL )))
		{
			firstpos = pos ;
		}
	}
	return firstpos ;
}

int parse_file_list ( char * list_with_spaces , char ** separated_list , int max_elements )
{
	int			i ;
	char		* pos ;
	char		* end ;
	char		* start ;
	char		* quote_char ;
	char		temp [ 2 ] ;
	int			num = 0 ;

	if ( strlen ( list_with_spaces ) < 1 )
	{
		return 0 ;
	}
	temp [ 1 ] = '\0' ;
	quote_char = NULL ;
	for ( i = 0 ; i < max_elements ; i ++ )
	{
		separated_list [ i ] = NULL ;
	}
	start = list_with_spaces ;
	while (( num < max_elements )
	&& ( pos = find_index_of_first ( start , 
			( quote_char != NULL ) ? quote_char : "\"' " )) != NULL )
	{
		if ( pos [ 0 ] != ' ' )
		{
			if ( quote_char != NULL )
			{
				/* end of quote */
				quote_char = NULL ;
			} else {
				/* start of quote */
				temp [ 0 ] = pos [ 0 ] ;
				quote_char = temp ;
			}
		}
		end = pos - 1 ;
		if ( end >= start )
		{
			separated_list [ num ] = malloc (( end - start ) + 2 ) ;
			memcpy ( separated_list [ num ] , start , ( end - start ) + 1 ) ;
			separated_list [ num ][ ( end - start ) + 1 ] = '\0' ;
			num ++ ;
		}
		start = pos + 1 ;
	}
	if (( strlen ( start ) > 0 )
	&& ( num < max_elements ))
	{
		separated_list [ num ] = malloc ( strlen ( start ) + 1 ) ;
		memcpy ( separated_list [ num ] , start , strlen ( start ) ) ;
		separated_list [ num ][ strlen ( start ) ] = '\0' ;
		num ++ ;
	}
	return num ;
}

void remove_trailing_slashes ( char * str )
{
	int		i ;

	for ( i = strlen ( str ) - 1 ; ( i >= 0 ) && ( str [ i ] == '/' ) ; i -- )
	{
		str [ i ] = '\0' ;
	}
}

void string_uppercase ( char * str )
{
	int		i ;

	for ( i = 0 ; i < strlen ( str ) ; i ++ )
	{
		str [ i ] = toupper ( str [ i ] ) ;
	}
}

void print_all_keys ( parser_table * t )
{
	int			i ;
	char		* width_str ;
	int			width , column = 0 ;

	width_str = getenv ( "COLUMNS" ) ;
	if ( width_str != NULL )
	{
		width = atoi ( width_str ) ;
	} else {
		width = 80 ;
	}
	if ( width < 25 )
	{
		width = 25 ;
	}

	for ( i = 0 ; strlen ( t [ i ] . key ) > 0 ; i ++ )
	{
		printf ( "%21s  " , t [ i ] . key ) ;
		column += 23 ;
		if ( ( column + 23 ) >= width )
		{
			printf ( "\n" ) ;
			column = 0 ;
		}
	}
}



