/*
 * player.h 
 *
 * Contains the player_info structure. Originally from dvdplay,
 * (C) Creative Labs 1999
 *
 * Changes in this version (C) Jack Whitham 2001
 */


#ifndef PLAYER_H
#define PLAYER_H


#include <errno.h>
#include <stdlib.h>
#include <pthread.h>
#include "ifo/libifo/ifo.h"
#include <linux/limits.h>

#include <config.h>

#ifndef HAVE_DVD_AUTHINFO
//#error "DVD authinfo is not available. This is essential for DVD playback. See the documentation."
#endif

#include "output.h"

#define FRAME_SIZE 						0x8000
#define CHUNK_SIZE 						0x800
#define ANNOUNCE						"minidxr 0.1"

#define BUFFER_LENGTH					1024
#define BUFFER_LENGTH_1					( BUFFER_LENGTH - 1 )
#define FNAME_LENGTH					BUFFER_LENGTH
#define FNAME_LENGTH_1					( FNAME_LENGTH - 1 )
#define SHORT_BUFFER_LENGTH				128
#define SHORT_BUFFER_LENGTH_1			( SHORT_BUFFER_LENGTH - 1 )

#define BUFFER_CHUNK_SHIFT				11		
#define BUFFER_CHUNK_SIZE				( 1 << BUFFER_CHUNK_SHIFT )		/* 2048 */
#define BUFFER_CHUNK_BITMASK			( BUFFER_CHUNK_SIZE - 1 )		/* 2047 */
#define BUFFER_RING_SIZE				512
#define SUBPROG_ERROR_CODE				"Error: "

#define MAX_IP_FILES					128 
#define IP_OFF							0
#define IP_FILES						1
#define IP_DVD							2


#define SAFE_STRNCPY(dst,src,len) \
				if ( dst != NULL ) \
				{ \
					strncpy ( ( dst ) , ( src ) , ( len ) ) ; \
					( dst ) [ ( len ) - 1 ] = '\0' ; \
				} 



typedef struct mfile_struct {
				  char* name;
				  int fd, size, lba;
				  int on_dvd ;
				  int start_pos , end_pos ;
				  int type ;
				} mfile ;

typedef struct play_stack_cell_struct
				{
					int					start , end ;
					int					cell_id ;
					struct play_stack_cell_struct	
										* next ;
				} play_stack_cell ;

typedef struct play_stack_node_struct
				{
					play_stack_cell		** cells ;
					int					num_cells ;
				} play_stack_node ;

typedef struct {
					char				data [ BUFFER_CHUNK_SIZE ] ;
					int					source_pos ;
					int					size ;
					int					source_lba ;
					int					source_type ;
				}  buffer_item ;

typedef struct buffer_type_struct {
					buffer_item			** the_buffer ;
					int					size ;
					int					number_in_buffer ;
					int					next_read_pos ;
					int					next_write_pos ;
				} buffer_type ;


typedef struct {
  /* setup */
  char uCode_file [ FNAME_LENGTH ] ;
  char dxr2_device [ FNAME_LENGTH ] ;
  char dvd_device [ FNAME_LENGTH ] ;
  char dvd_mountpoint [ FNAME_LENGTH ] ;
  char * out_file ;

  /* device */
  char * dvdtitleauth_path ;

  /* output */
  output_plugin * out_plugin ;

  /* control */
  int h_state ;
  int default_audio_format ;
  int default_audio_freq ;
  int default_audio_width ;
  int default_video_freq ;
  int default_tv_format ;
  int default_video_format ;
  output_extra_info default_colour_key ;
  int view_mode ;
  int view_mode_changeable ;

  /* buffer */
  mfile * file_ptrs ;
  int num_files ;
  int total_size ;
  buffer_type * buffer ;
  buffer_type * standby_buffer ;
  int write_position ;
  int read_position ;
  int read_position_sub ;
  int read_fd ;
  int writer_waiting ;
  int current_lba ;
  int current_type ;
  int b_running , b_quit ;
  int force_reauth ;
  int read_title ;
  pthread_mutex_t mutex;
  pthread_mutex_t file_access_mutex;
  pthread_mutex_t audio_mutex ;
  pthread_cond_t reader_state_change ;
  pthread_cond_t buffer_quite_full ;
  int recompute_end ;

  /* ifo */
  int current_title ;
  int current_stream ;

  /* ifo internals */
  int ifo_available ;
  ifo_t			* title_ifo [ 101 ] ;
  ifo_t			* disc_ifo ;
  int			title_ifo_fd [ 101 ] ;
  int			disc_ifo_fd ;
  int			num_physical_titles ;
  play_stack_node
				** play_stack ;
  int			play_stack_setup ;
  int			play_stack_num_vobs ;
  int			play_stack_title ;
  int			play_skip_angles ;
  int			play_num_angles ;
  int			play_stack_accelerator_cell ;
  int			play_stack_accelerator_vob ;
  pthread_mutex_t ifo_mutex;

  /* immediate playback support */
  int immediate_playback ;
  char * immediate_playback_names [ MAX_IP_FILES ] ;
  int immediate_playback_files ;

  /* message */
  buffer_type * message_queue ;
  pthread_mutex_t message_queue_mutex ;
} player_info_t;

extern int minidxr2_version_major ;
extern int minidxr2_version_minor ;

#endif




