/*
 * Minidxr2 - Linux DVD player. Mostly (C) Jack Whitham, 2001
 *
 */
/*
 * config.c
 *
 * Contains code to interpret the dxr2 config file.
 *
 */

#include <stdio.h>
#include <string.h>

#include "read_config.h"
#include "parser.h"
#include "output.h"


/* Config file names */

char	dxr2_config_file[] = "/etc/dxr2player.conf" ;
char	alt_config_file[] = "/etc/minidxr2.conf" ;

/* Config file interpreter tables */

parser_table audio_stream_table[] = 
		{ { "AC3" , (void *) OUTPUT_AUDIO_AC3 , NULL , INTEGER } ,
		{ "MPEG" , (void *) OUTPUT_AUDIO_MPEG , NULL , INTEGER } ,
		{ "PCM" , (void *) OUTPUT_AUDIO_LPCM , NULL , INTEGER } ,
		{ "LPCM" , (void *) OUTPUT_AUDIO_LPCM , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;
parser_table audio_freq_table[] = 
		{ { "44.1" , (void *) OUTPUT_AUDIO_FREQ_441 , NULL , INTEGER } ,
		{ "441" , (void *) OUTPUT_AUDIO_FREQ_441 , NULL , INTEGER } ,
		{ "44" , (void *) OUTPUT_AUDIO_FREQ_441 , NULL , INTEGER } ,
		{ "48" , (void *) OUTPUT_AUDIO_FREQ_48 , NULL , INTEGER } ,
		{ "96" , (void *) OUTPUT_AUDIO_FREQ_96 , NULL , INTEGER } ,
		{ "2205" , (void *) OUTPUT_AUDIO_FREQ_2205 , NULL , INTEGER } ,
		{ "22.05" , (void *) OUTPUT_AUDIO_FREQ_2205 , NULL , INTEGER } ,
		{ "22" , (void *) OUTPUT_AUDIO_FREQ_2205 , NULL , INTEGER } ,
		{ "32" , (void *) OUTPUT_AUDIO_FREQ_32 , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;
parser_table audio_width_table[] = 
		{ { "16" , (void *) OUTPUT_AUDIO_WIDTH_16 , NULL , INTEGER } ,
		{ "20" , (void *) OUTPUT_AUDIO_WIDTH_20 , NULL , INTEGER } ,
		{ "24" , (void *) OUTPUT_AUDIO_WIDTH_24 , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;
parser_table tv_format_table[] = { 
		{ "NTSC" , (void *) OUTPUT_FORMAT_NTSC , NULL , INTEGER } ,
		{ "NTSC60" , (void *) OUTPUT_FORMAT_NTSC_60 , NULL , INTEGER } ,
		{ "PAL_M" , (void *) OUTPUT_FORMAT_PAL_M , NULL , INTEGER } ,
		{ "PAL_M_60" , (void *) OUTPUT_FORMAT_PAL_M_60 , NULL , INTEGER } ,
		{ "PAL_BDGHI" , (void *) OUTPUT_FORMAT_PAL_BDGHI , NULL , INTEGER } ,
		{ "PAL_N" , (void *) OUTPUT_FORMAT_PAL_N , NULL , INTEGER } ,
		{ "PAL" , (void *) OUTPUT_FORMAT_PAL_N , NULL , INTEGER } ,
		{ "PAL_Nc" , (void *) OUTPUT_FORMAT_PAL_Nc , NULL , INTEGER } ,
		{ "PAL_60" , (void *) OUTPUT_FORMAT_PAL_60 , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;
parser_table video_freq_table[] = 
		{ { "30" , (void *) OUTPUT_SRC_VIDEO_FREQ_30 , NULL , INTEGER } ,
		{ "25" , (void *) OUTPUT_SRC_VIDEO_FREQ_25 , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;
parser_table video_format_table[] = 
		{ { "letterbox" , (void *) OUTPUT_ASPECTRATIOMODE_LETTERBOX , NULL , INTEGER } ,
		{ "normal" , (void *) OUTPUT_ASPECTRATIOMODE_NORMAL , NULL , INTEGER } ,
		{ "pan_scan" , (void *) OUTPUT_ASPECTRATIOMODE_PAN_SCAN , NULL , INTEGER } ,
		{ "" , NULL , NULL , 0 } } ;


char * read_config(player_info_t* player_info, char * config_file )
{
	FILE* config;
	char input[BUFFER_LENGTH];
	char * input2 ;
	char * err ;
	char * table_ptr ;
	char * output_ptr ;
	char overlay_crop [ BUFFER_LENGTH ] ;

	/* The general table must go here, since it has some dynamic members */

	parser_table general_table[] = {
		{ "drive" , player_info->dvd_device , NULL , STRING } ,
		{ "dxr2" , player_info->dxr2_device , NULL , STRING } ,
		{ "mountpoint" , player_info->dvd_mountpoint , NULL , STRING } ,
		{ "firmware" , player_info->uCode_file , NULL , STRING } ,
		{ "audio_format" , audio_stream_table , & player_info -> default_audio_format , TABLE } ,
		{ "audio_freq" , audio_freq_table , & player_info -> default_audio_freq , TABLE } ,
		{ "audio_width" , audio_width_table , & player_info -> default_audio_width , TABLE } ,
		{ "tv_format" , tv_format_table , & player_info -> default_tv_format , TABLE } ,
		{ "video_freq" , video_freq_table , & player_info -> default_video_freq , TABLE } , 
		{ "video_format" , video_format_table , & player_info -> default_video_format , TABLE } ,
		{ "subpicture" , NULL , NULL , NONE } ,
		{ "overlay" , NULL , NULL , NONE } ,
		{ "overlay_geom" , NULL , NULL , NONE } ,
		{ "overlay_crop" , overlay_crop , NULL , STRING } ,
		{ "red" , & player_info -> default_colour_key . red_min , 
			& player_info -> default_colour_key . red_max , INT_RANGE } ,
		{ "green" , & player_info -> default_colour_key . green_min , 
			& player_info -> default_colour_key . green_max , INT_RANGE } ,
		{ "blue" , & player_info -> default_colour_key . blue_min , 
			& player_info -> default_colour_key . blue_max , INT_RANGE } ,
		{ "" , NULL , NULL , 0 } } ;

	strcpy ( player_info->uCode_file , "/etc/dvd12.ux" ) ;
	strcpy ( player_info->dvd_mountpoint , "/cdrom" ) ;
	strcpy ( player_info->dvd_device , "/dev/cdrom" ) ;
	strcpy ( player_info->dxr2_device , "/dev/dxr2" ) ;
	overlay_crop [ 0 ] = '\0' ;

	config = fopen(config_file, "rt");

	if ( config == NULL )
	{
		err = "Error loading config file!\n" ;
		return err ;
	} else {
		err = malloc ( BUFFER_LENGTH ) ;

		while( fgets ( input , BUFFER_LENGTH_1 , config ) != NULL )
		{
			remove_whitespace ( input ) ;

			if (( strlen ( input ) > 0 )
			&& ( input [ 0 ] != '#' )) /* skip to end of line */
			{
				input2 = index ( input , ':' ) ;
				if ( input2 == NULL )
				{
					snprintf ( err , BUFFER_LENGTH_1 , "Cannot understand config file line beginning '%s'\n" ,
								input ) ;
					return err ;
				} else {
					input2 [ 0 ] = '\0' ;
					input2 = & input2 [ 1 ] ;
					remove_whitespace ( input2 ) ;
				}
				switch ( parse_type ( input , general_table , (void **) & table_ptr , (void **) & output_ptr , "identifier" , err ) )
				{
					case STRING :
							strcpy ( (char *) table_ptr , input2 ) ;
							break ;
					case TABLE :
							if ( parse_type ( input2 , (parser_table *) table_ptr ,
											(void **) output_ptr , NULL , input , err ) == 0 )
							{
								return err ;
							}
							break ;
					case INT_RANGE :
							{
								char		* dash ;
								char		* first_end ;
								char		* second_end ;
								int			first , second ;

								dash = index ( input2 , '-' ) ;
								if ( dash != NULL )
								{
									dash [ 0 ] = '\0' ;
									first = strtol ( input2 , & first_end , 16 ) ;
									second = strtol ( & dash [ 1 ] , & second_end , 16 ) ;
									if (( first_end != NULL )
									&& ( second_end != NULL )
									&& ( first >= 0 )
									&& ( first <= 255 )
									&& ( second >= 0 )
									&& ( second <= 255 ))
									{
										(* table_ptr) = first ;
										(* output_ptr) = second ;
									} else {
										snprintf ( err , BUFFER_LENGTH_1 , 
												"Cannot interpret range on config file line beginning '%s'\n" ,
													input ) ;
										return err ;
									}
								} else {
									snprintf ( err , BUFFER_LENGTH_1 ,
												"A hex range, such as 00-ff, was expected on config file line beginning '%s'\n" ,
													input ) ;
									return err ;
								}
							}
							break ;
					case NONE :
							break ;
					default :
							return err ;
				}
			}
		}
		free ( err ) ;
	}

	if ( strlen ( overlay_crop ) > 0 )
	{
		/* these values are sanity tested later. */
		sscanf ( overlay_crop , "%dx%dx%dx%d" ,
				& player_info -> default_colour_key . left_crop ,
				& player_info -> default_colour_key . right_crop ,
				& player_info -> default_colour_key . top_crop ,
				& player_info -> default_colour_key . bottom_crop ) ;
	}

	return NULL ;
}

